import importlib
import re

from proto.enums import ProtoEnumMeta

from google.ads.googleads import client

enums_module = importlib.import_module(
    f"google.ads.googleads.{client._DEFAULT_VERSION}.enums"
)


lines = []
for enum in enums_module.__all__:
    enum_class = getattr(enums_module, enum)
    for attr in dir(enum_class):
        attr_val = getattr(enum_class, attr)
        if isinstance(attr_val, ProtoEnumMeta):
            lines.append(
                f"    {enum}: type[{client._DEFAULT_VERSION}.enums.{enum}.{attr_val.__name__}]"
            )
            break

with open("google-stubs/ads/googleads/client.pyi") as f:
    client_pyi = f.read()
lines.insert(0, "# Autogenerated enums")
lines.append("    # End of autogenerated enums")
client_pyi = re.sub(
    "# Autogenerated enums.+# End of autogenerated enums",
    "\n".join(lines),
    client_pyi,
    flags=re.DOTALL,
)
with open("google-stubs/ads/googleads/client.pyi", mode="w") as f:
    f.write(client_pyi)



from .async_client import AdGroupFeedServiceAsyncClient as AdGroupFeedServiceAsyncClient
from .client import AdGroupFeedServiceClient as AdGroupFeedServiceClient

__all__ = ["AdGroupFeedServiceClient", "AdGroupFeedServiceAsyncClient"]



import types
from typing import Callable, Dict, Sequence, Tuple, Type

from _typeshed import Incomplete
from google.api_core import (
    client_options as client_options_lib,
    gapic_v1,
    retry as retries,
)
from google.auth import credentials as ga_credentials

from google.ads.googleads.v20.resources.types import google_ads_field
from google.ads.googleads.v20.services.services.google_ads_field_service import pagers
from google.ads.googleads.v20.services.types import google_ads_field_service

from .transports.base import GoogleAdsFieldServiceTransport

__all__ = ["GoogleAdsFieldServiceClient"]

class GoogleAdsFieldServiceClientMeta(type):
    def get_transport_class(
        cls, label: str | None = None
    ) -> type[GoogleAdsFieldServiceTransport]: ...

class GoogleAdsFieldServiceClient(metaclass=GoogleAdsFieldServiceClientMeta):
    DEFAULT_ENDPOINT: str
    DEFAULT_MTLS_ENDPOINT: Incomplete
    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs): ...
    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs): ...
    from_service_account_json = from_service_account_file
    @property
    def transport(self) -> GoogleAdsFieldServiceTransport: ...
    @staticmethod
    def google_ads_field_path(google_ads_field: str) -> str: ...
    @staticmethod
    def parse_google_ads_field_path(path: str) -> dict[str, str]: ...
    @staticmethod
    def common_billing_account_path(billing_account: str) -> str: ...
    @staticmethod
    def parse_common_billing_account_path(path: str) -> dict[str, str]: ...
    @staticmethod
    def common_folder_path(folder: str) -> str: ...
    @staticmethod
    def parse_common_folder_path(path: str) -> dict[str, str]: ...
    @staticmethod
    def common_organization_path(organization: str) -> str: ...
    @staticmethod
    def parse_common_organization_path(path: str) -> dict[str, str]: ...
    @staticmethod
    def common_project_path(project: str) -> str: ...
    @staticmethod
    def parse_common_project_path(path: str) -> dict[str, str]: ...
    @staticmethod
    def common_location_path(project: str, location: str) -> str: ...
    @staticmethod
    def parse_common_location_path(path: str) -> dict[str, str]: ...
    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: client_options_lib.ClientOptions | None = None
    ): ...
    @property
    def api_endpoint(self): ...
    @property
    def universe_domain(self) -> str: ...
    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials | None = None,
        transport: str
        | GoogleAdsFieldServiceTransport
        | Callable[..., GoogleAdsFieldServiceTransport]
        | None = None,
        client_options: client_options_lib.ClientOptions | dict | None = None,
        client_info: gapic_v1.client_info.ClientInfo = ...,
    ) -> None: ...
    def get_google_ads_field(
        self,
        request: google_ads_field_service.GetGoogleAdsFieldRequest | dict | None = None,
        *,
        resource_name: str | None = None,
        retry: retries.Retry | gapic_v1.method._MethodDefault = ...,
        timeout: float | object = ...,
        metadata: Sequence[tuple[str, str | bytes]] = (),
    ) -> google_ads_field.GoogleAdsField: ...
    def search_google_ads_fields(
        self,
        request: google_ads_field_service.SearchGoogleAdsFieldsRequest
        | dict
        | None = None,
        *,
        query: str | None = None,
        retry: retries.Retry | gapic_v1.method._MethodDefault = ...,
        timeout: float | object = ...,
        metadata: Sequence[tuple[str, str | bytes]] = (),
    ) -> pagers.SearchGoogleAdsFieldsPager: ...
    def __enter__(self) -> GoogleAdsFieldServiceClient: ...
    def __exit__(
        self,
        type: type[BaseException] | None,
        value: BaseException | None,
        traceback: types.TracebackType | None,
    ) -> None: ...



from .base import FeedItemSetLinkServiceTransport as FeedItemSetLinkServiceTransport
from .grpc import (
    FeedItemSetLinkServiceGrpcTransport as FeedItemSetLinkServiceGrpcTransport,
)
from .grpc_asyncio import (
    FeedItemSetLinkServiceGrpcAsyncIOTransport as FeedItemSetLinkServiceGrpcAsyncIOTransport,
)

__all__ = [
    "FeedItemSetLinkServiceTransport",
    "FeedItemSetLinkServiceGrpcTransport",
    "FeedItemSetLinkServiceGrpcAsyncIOTransport",
]



from .base import AdGroupFeedServiceTransport as AdGroupFeedServiceTransport
from .grpc import AdGroupFeedServiceGrpcTransport as AdGroupFeedServiceGrpcTransport
from .grpc_asyncio import (
    AdGroupFeedServiceGrpcAsyncIOTransport as AdGroupFeedServiceGrpcAsyncIOTransport,
)

__all__ = [
    "AdGroupFeedServiceTransport",
    "AdGroupFeedServiceGrpcTransport",
    "AdGroupFeedServiceGrpcAsyncIOTransport",
]



from .base import FeedServiceTransport as FeedServiceTransport
from .grpc import FeedServiceGrpcTransport as FeedServiceGrpcTransport
from .grpc_asyncio import (
    FeedServiceGrpcAsyncIOTransport as FeedServiceGrpcAsyncIOTransport,
)

__all__ = [
    "FeedServiceTransport",
    "FeedServiceGrpcTransport",
    "FeedServiceGrpcAsyncIOTransport",
]



from .async_client import (
    CustomerExtensionSettingServiceAsyncClient as CustomerExtensionSettingServiceAsyncClient,
)
from .client import (
    CustomerExtensionSettingServiceClient as CustomerExtensionSettingServiceClient,
)

__all__ = [
    "CustomerExtensionSettingServiceClient",
    "CustomerExtensionSettingServiceAsyncClient",
]



from .base import FeedItemTargetServiceTransport as FeedItemTargetServiceTransport
from .grpc import (
    FeedItemTargetServiceGrpcTransport as FeedItemTargetServiceGrpcTransport,
)
from .grpc_asyncio import (
    FeedItemTargetServiceGrpcAsyncIOTransport as FeedItemTargetServiceGrpcAsyncIOTransport,
)

__all__ = [
    "FeedItemTargetServiceTransport",
    "FeedItemTargetServiceGrpcTransport",
    "FeedItemTargetServiceGrpcAsyncIOTransport",
]



from .async_client import (
    FeedItemSetLinkServiceAsyncClient as FeedItemSetLinkServiceAsyncClient,
)
from .client import FeedItemSetLinkServiceClient as FeedItemSetLinkServiceClient

__all__ = ["FeedItemSetLinkServiceClient", "FeedItemSetLinkServiceAsyncClient"]



from .base import FeedMappingServiceTransport as FeedMappingServiceTransport
from .grpc import FeedMappingServiceGrpcTransport as FeedMappingServiceGrpcTransport
from .grpc_asyncio import (
    FeedMappingServiceGrpcAsyncIOTransport as FeedMappingServiceGrpcAsyncIOTransport,
)

__all__ = [
    "FeedMappingServiceTransport",
    "FeedMappingServiceGrpcTransport",
    "FeedMappingServiceGrpcAsyncIOTransport",
]



from .base import ExtensionFeedItemServiceTransport as ExtensionFeedItemServiceTransport
from .grpc import (
    ExtensionFeedItemServiceGrpcTransport as ExtensionFeedItemServiceGrpcTransport,
)
from .grpc_asyncio import (
    ExtensionFeedItemServiceGrpcAsyncIOTransport as ExtensionFeedItemServiceGrpcAsyncIOTransport,
)

__all__ = [
    "ExtensionFeedItemServiceTransport",
    "ExtensionFeedItemServiceGrpcTransport",
    "ExtensionFeedItemServiceGrpcAsyncIOTransport",
]



import os
import shutil
import subprocess
from pathlib import Path


def copytree(src, dst, symlinks=False, ignore=None, overwrite=True):
    if not os.path.exists(dst):
        os.makedirs(dst)
    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            copytree(s, d, symlinks, ignore, overwrite)
        else:
            if not os.path.exists(d) or (
                overwrite and os.stat(s).st_mtime - os.stat(d).st_mtime > 1
            ):
                shutil.copy2(s, d)


# TODO: Explore options? E.g. --include-docstrings
subprocess.run(["stubgen", "google", "-o", ".."], cwd="google-ads-python", check=True)
copytree("googleads", "google-stubs/ads/googleads/", overwrite=False)
shutil.rmtree("googleads")
for dir, _, _ in os.walk("google-stubs/ads/googleads"):
    Path(dir, "__init__.pyi").touch()



from .async_client import (
    CampaignFeedServiceAsyncClient as CampaignFeedServiceAsyncClient,
)
from .client import CampaignFeedServiceClient as CampaignFeedServiceClient

__all__ = ["CampaignFeedServiceClient", "CampaignFeedServiceAsyncClient"]



import grpc

from google.ads.googleads.v18.errors import GoogleAdsFailure as GoogleAdsFailureV18
from google.ads.googleads.v19.errors import GoogleAdsFailure as GoogleAdsFailureV19
from google.ads.googleads.v20.errors import GoogleAdsFailure as GoogleAdsFailureV20

GoogleAdsFailure = GoogleAdsFailureV18 | GoogleAdsFailureV19 | GoogleAdsFailureV20

class GoogleAdsException(Exception):
    error: grpc.RpcError = ...
    call: grpc.Call = ...
    failure: GoogleAdsFailure = ...
    request_id: str = ...
    def __init__(
        self,
        error: grpc.RpcError,
        call: grpc.Call,
        failure: GoogleAdsFailure,
        request_id: str,
    ) -> None: ...



from .async_client import (
    FeedItemTargetServiceAsyncClient as FeedItemTargetServiceAsyncClient,
)
from .client import FeedItemTargetServiceClient as FeedItemTargetServiceClient

__all__ = ["FeedItemTargetServiceClient", "FeedItemTargetServiceAsyncClient"]



from .async_client import (
    CustomerFeedServiceAsyncClient as CustomerFeedServiceAsyncClient,
)
from .client import CustomerFeedServiceClient as CustomerFeedServiceClient

__all__ = ["CustomerFeedServiceClient", "CustomerFeedServiceAsyncClient"]



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v19.enums.types.advertising_channel_type import (
    AdvertisingChannelTypeEnum,
)
from google.ads.googleads.v19.enums.types.asset_field_type import AssetFieldTypeEnum
from google.ads.googleads.v19.enums.types.asset_source import AssetSourceEnum

_M = TypeVar("_M")

class ChannelAggregateAssetView(proto.Message):
    resource_name: str
    advertising_channel_type: AdvertisingChannelTypeEnum.AdvertisingChannelType
    asset: str
    asset_source: AssetSourceEnum.AssetSource
    field_type: AssetFieldTypeEnum.AssetFieldType
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        advertising_channel_type: AdvertisingChannelTypeEnum.AdvertisingChannelType = ...,
        asset: str = ...,
        asset_source: AssetSourceEnum.AssetSource = ...,
        field_type: AssetFieldTypeEnum.AssetFieldType = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "resource_name",
            "advertising_channel_type",
            "asset",
            "asset_source",
            "field_type",
        ],
    ) -> bool: ...



from .base import CustomerFeedServiceTransport as CustomerFeedServiceTransport
from .grpc import CustomerFeedServiceGrpcTransport as CustomerFeedServiceGrpcTransport
from .grpc_asyncio import (
    CustomerFeedServiceGrpcAsyncIOTransport as CustomerFeedServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CustomerFeedServiceTransport",
    "CustomerFeedServiceGrpcTransport",
    "CustomerFeedServiceGrpcAsyncIOTransport",
]



from .async_client import (
    CampaignExtensionSettingServiceAsyncClient as CampaignExtensionSettingServiceAsyncClient,
)
from .client import (
    CampaignExtensionSettingServiceClient as CampaignExtensionSettingServiceClient,
)

__all__ = [
    "CampaignExtensionSettingServiceClient",
    "CampaignExtensionSettingServiceAsyncClient",
]



from .async_client import (
    ExtensionFeedItemServiceAsyncClient as ExtensionFeedItemServiceAsyncClient,
)
from .client import ExtensionFeedItemServiceClient as ExtensionFeedItemServiceClient

__all__ = ["ExtensionFeedItemServiceClient", "ExtensionFeedItemServiceAsyncClient"]



from .base import FeedItemSetServiceTransport as FeedItemSetServiceTransport
from .grpc import FeedItemSetServiceGrpcTransport as FeedItemSetServiceGrpcTransport
from .grpc_asyncio import (
    FeedItemSetServiceGrpcAsyncIOTransport as FeedItemSetServiceGrpcAsyncIOTransport,
)

__all__ = [
    "FeedItemSetServiceTransport",
    "FeedItemSetServiceGrpcTransport",
    "FeedItemSetServiceGrpcAsyncIOTransport",
]



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v20.enums.types.conversion_value_rule_status import (
    ConversionValueRuleStatusEnum,
)
from google.ads.googleads.v20.enums.types.value_rule_device_type import (
    ValueRuleDeviceTypeEnum,
)
from google.ads.googleads.v20.enums.types.value_rule_geo_location_match_type import (
    ValueRuleGeoLocationMatchTypeEnum,
)
from google.ads.googleads.v20.enums.types.value_rule_operation import (
    ValueRuleOperationEnum,
)

_M = TypeVar("_M")

class ConversionValueRule(proto.Message):
    class ValueRuleAction(proto.Message):
        operation: ValueRuleOperationEnum.ValueRuleOperation
        value: float
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            operation: ValueRuleOperationEnum.ValueRuleOperation = ...,
            value: float = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["operation", "value"]
        ) -> bool: ...

    class ValueRuleAudienceCondition(proto.Message):
        user_lists: MutableSequence[str]
        user_interests: MutableSequence[str]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            user_lists: MutableSequence[str] = ...,
            user_interests: MutableSequence[str] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["user_lists", "user_interests"]
        ) -> bool: ...

    class ValueRuleDeviceCondition(proto.Message):
        device_types: MutableSequence[ValueRuleDeviceTypeEnum.ValueRuleDeviceType]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            device_types: MutableSequence[
                ValueRuleDeviceTypeEnum.ValueRuleDeviceType
            ] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["device_types"]
        ) -> bool: ...

    class ValueRuleGeoLocationCondition(proto.Message):
        excluded_geo_target_constants: MutableSequence[str]
        excluded_geo_match_type: (
            ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType
        )
        geo_target_constants: MutableSequence[str]
        geo_match_type: ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            excluded_geo_target_constants: MutableSequence[str] = ...,
            excluded_geo_match_type: ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType = ...,
            geo_target_constants: MutableSequence[str] = ...,
            geo_match_type: ValueRuleGeoLocationMatchTypeEnum.ValueRuleGeoLocationMatchType = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self,
            key: Literal[
                "excluded_geo_target_constants",
                "excluded_geo_match_type",
                "geo_target_constants",
                "geo_match_type",
            ],
        ) -> bool: ...

    class ValueRuleItineraryAdvanceBookingWindow(proto.Message):
        min_days: int
        max_days: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            min_days: int = ...,
            max_days: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["min_days", "max_days"]
        ) -> bool: ...

    class ValueRuleItineraryCondition(proto.Message):
        advance_booking_window: (
            ConversionValueRule.ValueRuleItineraryAdvanceBookingWindow
        )
        travel_length: ConversionValueRule.ValueRuleItineraryTravelLength
        travel_start_day: ConversionValueRule.ValueRuleItineraryTravelStartDay
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            advance_booking_window: ConversionValueRule.ValueRuleItineraryAdvanceBookingWindow = ...,
            travel_length: ConversionValueRule.ValueRuleItineraryTravelLength = ...,
            travel_start_day: ConversionValueRule.ValueRuleItineraryTravelStartDay = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self,
            key: Literal["advance_booking_window", "travel_length", "travel_start_day"],
        ) -> bool: ...

    class ValueRuleItineraryTravelLength(proto.Message):
        min_nights: int
        max_nights: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            min_nights: int = ...,
            max_nights: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["min_nights", "max_nights"]
        ) -> bool: ...

    class ValueRuleItineraryTravelStartDay(proto.Message):
        monday: bool
        tuesday: bool
        wednesday: bool
        thursday: bool
        friday: bool
        saturday: bool
        sunday: bool
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            monday: bool = ...,
            tuesday: bool = ...,
            wednesday: bool = ...,
            thursday: bool = ...,
            friday: bool = ...,
            saturday: bool = ...,
            sunday: bool = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self,
            key: Literal[
                "monday",
                "tuesday",
                "wednesday",
                "thursday",
                "friday",
                "saturday",
                "sunday",
            ],
        ) -> bool: ...

    resource_name: str
    id: int
    action: ConversionValueRule.ValueRuleAction
    geo_location_condition: ConversionValueRule.ValueRuleGeoLocationCondition
    device_condition: ConversionValueRule.ValueRuleDeviceCondition
    audience_condition: ConversionValueRule.ValueRuleAudienceCondition
    itinerary_condition: ConversionValueRule.ValueRuleItineraryCondition
    owner_customer: str
    status: ConversionValueRuleStatusEnum.ConversionValueRuleStatus
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        id: int = ...,
        action: ConversionValueRule.ValueRuleAction = ...,
        geo_location_condition: ConversionValueRule.ValueRuleGeoLocationCondition = ...,
        device_condition: ConversionValueRule.ValueRuleDeviceCondition = ...,
        audience_condition: ConversionValueRule.ValueRuleAudienceCondition = ...,
        itinerary_condition: ConversionValueRule.ValueRuleItineraryCondition = ...,
        owner_customer: str = ...,
        status: ConversionValueRuleStatusEnum.ConversionValueRuleStatus = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "resource_name",
            "id",
            "action",
            "geo_location_condition",
            "device_condition",
            "audience_condition",
            "itinerary_condition",
            "owner_customer",
            "status",
        ],
    ) -> bool: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from google.protobuf.field_mask_pb2 import FieldMask
from google.rpc.status_pb2 import Status
from typing_extensions import Literal

from google.ads.googleads.v19.common.types.policy import PolicyValidationParameter
from google.ads.googleads.v19.enums.types.asset_field_type import AssetFieldTypeEnum
from google.ads.googleads.v19.enums.types.response_content_type import (
    ResponseContentTypeEnum,
)
from google.ads.googleads.v19.resources.types.ad_group_ad import AdGroupAd

_M = TypeVar("_M")

class AdGroupAdOperation(proto.Message):
    update_mask: FieldMask
    policy_validation_parameter: PolicyValidationParameter
    create: AdGroupAd
    update: AdGroupAd
    remove: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        update_mask: FieldMask = ...,
        policy_validation_parameter: PolicyValidationParameter = ...,
        create: AdGroupAd = ...,
        update: AdGroupAd = ...,
        remove: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "update_mask", "policy_validation_parameter", "create", "update", "remove"
        ],
    ) -> bool: ...

class AssetsWithFieldType(proto.Message):
    asset: str
    asset_field_type: AssetFieldTypeEnum.AssetFieldType
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        asset: str = ...,
        asset_field_type: AssetFieldTypeEnum.AssetFieldType = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["asset", "asset_field_type"]
    ) -> bool: ...

class MutateAdGroupAdResult(proto.Message):
    resource_name: str
    ad_group_ad: AdGroupAd
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        ad_group_ad: AdGroupAd = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["resource_name", "ad_group_ad"]
    ) -> bool: ...

class MutateAdGroupAdsRequest(proto.Message):
    customer_id: str
    operations: MutableSequence[AdGroupAdOperation]
    partial_failure: bool
    validate_only: bool
    response_content_type: ResponseContentTypeEnum.ResponseContentType
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        operations: MutableSequence[AdGroupAdOperation] = ...,
        partial_failure: bool = ...,
        validate_only: bool = ...,
        response_content_type: ResponseContentTypeEnum.ResponseContentType = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "customer_id",
            "operations",
            "partial_failure",
            "validate_only",
            "response_content_type",
        ],
    ) -> bool: ...

class MutateAdGroupAdsResponse(proto.Message):
    partial_failure_error: Status
    results: MutableSequence[MutateAdGroupAdResult]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        partial_failure_error: Status = ...,
        results: MutableSequence[MutateAdGroupAdResult] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["partial_failure_error", "results"]
    ) -> bool: ...

class RemoveAutomaticallyCreatedAssetsRequest(proto.Message):
    ad_group_ad: str
    assets_with_field_type: MutableSequence[AssetsWithFieldType]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        ad_group_ad: str = ...,
        assets_with_field_type: MutableSequence[AssetsWithFieldType] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["ad_group_ad", "assets_with_field_type"]
    ) -> bool: ...



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v20.enums.types.mobile_device_type import MobileDeviceTypeEnum

_M = TypeVar("_M")

class MobileDeviceConstant(proto.Message):
    resource_name: str
    id: int
    name: str
    manufacturer_name: str
    operating_system_name: str
    type_: MobileDeviceTypeEnum.MobileDeviceType
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        id: int = ...,
        name: str = ...,
        manufacturer_name: str = ...,
        operating_system_name: str = ...,
        type_: MobileDeviceTypeEnum.MobileDeviceType = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "resource_name",
            "id",
            "name",
            "manufacturer_name",
            "operating_system_name",
            "type_",
        ],
    ) -> bool: ...



from .async_client import FeedMappingServiceAsyncClient as FeedMappingServiceAsyncClient
from .client import FeedMappingServiceClient as FeedMappingServiceClient

__all__ = ["FeedMappingServiceClient", "FeedMappingServiceAsyncClient"]



import importlib
import re
from pathlib import Path

from google.ads.googleads import client

overload_lines = []
import_lines = []
for i, version in enumerate(sorted(client._VALID_API_VERSIONS)):

    for service_package_path in Path("google-ads-python/google/ads/googleads", version, "services/services").iterdir():
        if service_package_path.name == "__init__.py":
            continue
        import_path = f"google.ads.googleads.{version}.services.services.{service_package_path.name}"
        service_package = importlib.import_module(import_path)
        import_lines.append(f"import {import_path}")
        for service_name in service_package.__all__:
            service_name_without_suffix = service_name[: -len("Client")]
            overload_lines.append(
                f'    @overload\n    def get_service(self, name: Literal["{service_name_without_suffix}"], version: _{version.upper()}) -> {import_path}.{service_name}: ...'
            )
            if version == client._DEFAULT_VERSION:
                overload_lines.append(
                    f'    @overload\n    def get_service(self, name: Literal["{service_name_without_suffix}"]) -> {import_path}.{service_name}: ...'
                )

    if i < len(client._VALID_API_VERSIONS) - 1:
        overload_lines.append("\n")

overload_lines.append(
    f'    @overload\n    def get_service(self, name: str, version: _V = "{client._DEFAULT_VERSION}") -> Any: ...'
)

with open("google-stubs/ads/googleads/client.pyi") as f:
    client_pyi = f.read()
overload_lines.insert(0, "# Autogenerated service overloads")
overload_lines.append("    # End of autogenerated service overloads")
client_pyi = re.sub(
    "# Autogenerated service overloads.+# End of autogenerated service overloads",
    "\n".join(overload_lines),
    client_pyi,
    flags=re.DOTALL,
)
import_lines.insert(0, "# Autogenerated service imports")
import_lines.append("# End of autogenerated service imports")
client_pyi = re.sub(
    "# Autogenerated service imports.+# End of autogenerated service imports",
    "\n".join(import_lines),
    client_pyi,
    flags=re.DOTALL,
)
with open("google-stubs/ads/googleads/client.pyi", mode="w") as f:
    f.write(client_pyi)



from .base import CampaignFeedServiceTransport as CampaignFeedServiceTransport
from .grpc import CampaignFeedServiceGrpcTransport as CampaignFeedServiceGrpcTransport
from .grpc_asyncio import (
    CampaignFeedServiceGrpcAsyncIOTransport as CampaignFeedServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CampaignFeedServiceTransport",
    "CampaignFeedServiceGrpcTransport",
    "CampaignFeedServiceGrpcAsyncIOTransport",
]



from .base import (
    CustomerClientLinkServiceTransport as CustomerClientLinkServiceTransport,
)
from .grpc import (
    CustomerClientLinkServiceGrpcTransport as CustomerClientLinkServiceGrpcTransport,
)
from .grpc_asyncio import (
    CustomerClientLinkServiceGrpcAsyncIOTransport as CustomerClientLinkServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CustomerClientLinkServiceTransport",
    "CustomerClientLinkServiceGrpcTransport",
    "CustomerClientLinkServiceGrpcAsyncIOTransport",
]



from .async_client import (
    AdGroupExtensionSettingServiceAsyncClient as AdGroupExtensionSettingServiceAsyncClient,
)
from .client import (
    AdGroupExtensionSettingServiceClient as AdGroupExtensionSettingServiceClient,
)

__all__ = [
    "AdGroupExtensionSettingServiceClient",
    "AdGroupExtensionSettingServiceAsyncClient",
]



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v20.common.types.criterion_category_availability import (
    CriterionCategoryAvailability,
)
from google.ads.googleads.v20.enums.types.user_interest_taxonomy_type import (
    UserInterestTaxonomyTypeEnum,
)

_M = TypeVar("_M")

class UserInterest(proto.Message):
    resource_name: str
    taxonomy_type: UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType
    user_interest_id: int
    name: str
    user_interest_parent: str
    launched_to_all: bool
    availabilities: MutableSequence[CriterionCategoryAvailability]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        taxonomy_type: UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType = ...,
        user_interest_id: int = ...,
        name: str = ...,
        user_interest_parent: str = ...,
        launched_to_all: bool = ...,
        availabilities: MutableSequence[CriterionCategoryAvailability] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "resource_name",
            "taxonomy_type",
            "user_interest_id",
            "name",
            "user_interest_parent",
            "launched_to_all",
            "availabilities",
        ],
    ) -> bool: ...



from .async_client import FeedItemSetServiceAsyncClient as FeedItemSetServiceAsyncClient
from .client import FeedItemSetServiceClient as FeedItemSetServiceClient

__all__ = ["FeedItemSetServiceClient", "FeedItemSetServiceAsyncClient"]



from .base import FeedItemServiceTransport as FeedItemServiceTransport
from .grpc import FeedItemServiceGrpcTransport as FeedItemServiceGrpcTransport
from .grpc_asyncio import (
    FeedItemServiceGrpcAsyncIOTransport as FeedItemServiceGrpcAsyncIOTransport,
)

__all__ = [
    "FeedItemServiceTransport",
    "FeedItemServiceGrpcTransport",
    "FeedItemServiceGrpcAsyncIOTransport",
]



from .async_client import FeedItemServiceAsyncClient as FeedItemServiceAsyncClient
from .client import FeedItemServiceClient as FeedItemServiceClient

__all__ = ["FeedItemServiceClient", "FeedItemServiceAsyncClient"]



#!/usr/bin/env bash

import importlib
import inspect
from contextlib import contextmanager
from pathlib import Path

import proto


class Writer:
    def __init__(self, file):
        self.file = file
        self.lines = []
        self._indent_level = 0

    @contextmanager
    def indent(self):
        self._indent_level += 1
        yield
        self._indent_level -= 1

    def write(self, s: str = "", end="\n", indent=True, prepend=False):
        if not isinstance(s, str):
            s = str(s)
        if indent:
            s = " " * self._indent_level * 4 + s
        if prepend:
            self.lines.insert(0, s + end)
        else:
            self.lines.append(s + end)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc, exc_tb):
        if exc_type is None:
            self.file.writelines(self.lines)


def print_message(cls_name, cls, writer, imports):
    writer.write(f"class {cls_name}(proto.Message):")
    with writer.indent():
        for inner_cls_name, inner_cls in inspect.getmembers(
            cls,
            lambda value: inspect.isclass(value) and issubclass(value, proto.Message),
        ):
            if inner_cls_name != "__base__":
                print_message(inner_cls_name, inner_cls, writer, imports)

        for inner_enum_name, inner_enum in inspect.getmembers(
            cls, lambda value: inspect.isclass(value) and issubclass(value, proto.Enum)
        ):
            writer.write(f"class {inner_enum_name}(proto.Enum):")
            with writer.indent():
                for member_name, member in inner_enum.__members__.items():
                    writer.write(f"{member_name} = {member.value}")

        fields = {}
        for field_name, field in cls.meta.fields.items():
            if field.proto_type in [proto.DOUBLE, proto.FLOAT]:
                type = "float"
            elif field.proto_type in [
                proto.INT64,
                proto.UINT64,
                proto.INT32,
                proto.FIXED64,
                proto.FIXED32,
                proto.UINT32,
                proto.SFIXED32,
                proto.SFIXED64,
                proto.SINT32,
                proto.SINT64,
            ]:
                type = "int"
            elif field.proto_type == proto.BOOL:
                type = "bool"
            elif field.proto_type == proto.STRING:
                type = "str"
            elif field.proto_type == proto.BYTES:
                type = "bytes"
            elif field.proto_type == proto.MESSAGE:
                if isinstance(field.message, str):
                    type = field.message
                else:
                    type = field.message.__qualname__
                    if field.message.__module__ != cls.__module__:
                        imports.append(
                            f"from {field.message.__module__} import {field.message.__qualname__.split('.')[0]}"
                        )
            elif field.proto_type == proto.ENUM:
                if isinstance(field.message, str):
                    type = field.enum
                else:
                    type = field.enum.__qualname__
                    if field.enum.__module__ != cls.__module__:
                        imports.append(
                            f"from {field.enum.__module__} import {field.enum.__qualname__.split('.')[0]}"
                        )
            else:
                raise Exception(field.proto_type)
            if field.repeated:
                type = f"MutableSequence[{type}]"
                imports.append("from collections.abc import MutableSequence")
            fields[field_name] = type
            writer.write(f"{field_name}: {type}")

        writer.write(
            f"def __init__(self: _M, mapping: _M | Mapping | google.protobuf.message.Message | None = ..., *, ignore_unknown_fields: bool = ..., {', '.join(f'{field_name}: {type} = ...' for field_name, type in fields.items())}) -> None: ..."
        )
        quoted_fields = [f'"{field}"' for field in fields]
        writer.write(f"def __contains__(  # type: ignore[override]")
        if quoted_fields:
            writer.write(
                f"self, key: Literal[{', '.join(quoted_fields)}]) -> bool: ..."
            )
        else:
            writer.write(f"self, key: NoReturn) -> bool: ...")


for path in Path("./google-ads-python/google/ads/googleads/").glob("**/types/*.py"):
    if path.stem == "__init__":
        continue
    module = importlib.import_module(".".join([*path.parent.parts[1:], path.stem]))
    with open(
        Path("google-stubs", *path.parent.parts[2:], f"{path.stem}.pyi"), "w"
    ) as file, Writer(file) as writer:
        writer.write("import proto")
        writer.write("import google.protobuf.message")
        writer.write("from typing import Any, TypeVar, NoReturn")
        writer.write("from typing_extensions import Literal")
        writer.write("from collections.abc import Mapping")
        writer.write('_M = TypeVar("_M")')
        imports = []
        for cls_name, cls in inspect.getmembers(
            module,
            lambda value: inspect.isclass(value) and issubclass(value, proto.Message),
        ):
            print_message(cls_name, cls, writer, imports)
        for import_ in imports:
            writer.write(import_, prepend=True)
